##==========================================================================
## icp - interactive custom plots for R
## Package version: 1.1-0
##==========================================================================
##
## $Id: icustom.R 16 2008-07-01 20:16:53Z urbanek $
##
##--- custom plots ---

## .ic.plots: this environment is the root of all custom plots
## * each type of a custom plot has one entry in this environemnt, keyed
##   by the plot type name
## * each such entry is a list consisting of
##    plots     - an environment holding all instances
##    construct - custom plot call-back function
## * each instance is an environemnt containing at least following vars
##    plot      - plot object
##    construct - custom plot call-back function
##    data      - copy of the data passed to the instance
##    ...       - all initial parameters are assigned here
##    ...       - all call parameters are assigned here
.ic.plots <- new.env(baseenv())

## functions that will be exported to custom function's env
.ic.fns <- c("sobj.rect","sobj.point","sobj.polygon","sobj.reset",
             "iaxis","iaxis.reset.all")

.ivar.valid.name <- function(x) x

icustom.plot<-function(name, construct, preflight, postflight, min.data.dim=1, param=list()) {
  name <- gsub("[\\\\\\\"]"," ",as.character(name)[1])
  .ic.plots[[name]] <- list(plots=new.env(baseenv()),construct=construct)
  if (missing(preflight)) preflight <- NULL
  if (missing(postflight)) postflight <- NULL
  ## real params: vars, ..., name
  f<-function() {
    custom.env <- new.env(TRUE, emptyenv())
    title <- name
    if (!is.null(preflight)) { vv <- preflight(data=vars, env=custom.env, ...); if (!is.null(vv)) vars <- vv }
    vv<-vector()
    if (!exists(".ivar.valid.name")) .ivar.valid.name<-function(x) x
    for (i in 1:length(vars)) { 
      varname <- names(vars)[i]
      var <- vars[[i]]
      if (!is.null(varname))
        var <- ivar.new(.ivar.valid.name(varname), var)
      else
        var <- ivar.new(.ivar.valid.name("customVar"), var)
      if (inherits(var,"ivar"))  vv <- c(vv,var@vid)
    }
    ## get ID for the next plot or "1" if the environemnt is empty
    id<-1
    if (length(ls(.ic.plots[[name]]$plots))>0)
      id<-max(as.integer(ls(.ic.plots[[name]]$plots)))+1
    
    lastPlot<-.jcall(.icp.framework, "Lorg/rosuda/icustom/CustomCanvas;",
                     "newCustomplot", as.integer(vv),
                     paste('.ic.plots[["',name,'"]]',sep=''), as.character(id))
.vv<<-vv
.id<<-id
.pobj<<-lastPlot    
if (is.jnull(lastPlot)) stop("cannot create custom plot")
    if (is.character(title) && length(title)) .jcall(lastPlot,"V","setTitle",title[1])
    a<-.iplot.new(lastPlot,"icustom");
.plot<<-a
    e<-new.env(parent=.GlobalEnv)
    e$ready <- FALSE
    environment(construct) <- e
    for (f in .ic.fns) { assign(f,get(f),e); environment(e[[f]])<-e }
    e$construct <- construct
    e$.construct <- function(...) { iaxis.reset.all(); sobj.reset(); construct(...) }
    environment(e$.construct) <- e
    e$plot <- a
    e$data <- vars 
    e$env  <- custom.env
    for (x in names(param)) e[[x]] <- param[[x]]
    if (length(list(...))>0)
      { l<-list(...); for (i in names(l)) e[[i]] <- l[[i]] }
    .ic.plots[[name]]$plots[[as.character(id)]] <- e
    e$ready <- TRUE
    if (!is.null(postflight)) postflight(data=vars$data, plot=a, env=e$env, ...)
    a
  }
  formals(f)<-c(alist(vars=,...=),list(name=name))
  f
}
