##========== global sobj functions ============

sobj.reset <- function(p=plot) {
  .jcall(p$obj,,"resetPP")
}


##================ sobjects ===================

## (ids) case IDs (row numbers) of the cases to represent
## (x,y) top left corner of rectangle
## (w,h) width, height of rectangle
## (orientation) specifies the directon of the highlighting
sobj.rect<-function(ids, x, y, w, h, orientation="up") {
  or <- pmatch(orientation[1],c("up","right","down","left")) - 1
  if (is.na(or)) stop("Invalid orientation.")
  pp <- .jnew("org/rosuda/ibase/toolkit/PPrimRectangle",as.integer(or))
  if (w<0) { x<-x+w; w<--w }
  if (h<0) { y<-y+h; h<--h }
  .jcall(pp,,"setBounds",as.double(x),as.double(y),as.double(abs(w)),as.double(abs(h)))
  ## Java IDs are 0-based, R's are 1-based
  .jcall(pp,,"setCaseIDs",.jarray(as.integer(ids - 1)))
  .iplots.cp.addPP(plot,pp)
  pp
}

sobj.point <- function(ids, x, y, diam=3, border=FALSE, angle.hilite=FALSE, col=NULL) {
  pp <- .jnew("org/rosuda/ibase/toolkit/PPrimCircle")
  .jcall(pp,,"move",as.integer(x),as.integer(y))
  .jcall(pp,,"setDiam",as.integer(diam))
  if (border) pp$drawBorder <- TRUE
  if (angle.hilite) pp$brushClockwise <- TRUE
  .jcall(pp,,"setCaseIDs",.jarray(as.integer(ids - 1)))
  if (!is.null(col)) {
    pp$fillColor <- col
    pp$col <- col
  }
  .iplots.cp.addPP(plot,pp)
  pp
}

sobj.polygon <- function(ids, x, y, border=TRUE, closed=TRUE, fill=TRUE, fill.col=NULL) {
  pp <- .jnew("org/rosuda/ibase/toolkit/PPrimPolygon")
  if (length(x)<length(y)) x <- rep(x, length.out=length(y))
  if (length(y)<length(x)) y <- rep(y, length.out=length(x))
  pol <- .jnew("java/awt/Polygon",as.integer(x),as.integer(y),as.integer(length(x)))
  .jcall(pp,,"setPolygon", pol)
  if (!border) .jfield(pp, "drawBorder") <- border
  if (!closed) .jfield(pp, "closed") <- closed
  if (!fill) .jfield(pp, "fill") <- FALSE
  if (!is.null(fill.col)) {
    pp$fillColor=fill.col
    pp$col=fill.col
  }
  .jcall(pp,,"setCaseIDs",.jarray(as.integer(ids - 1)))
  .iplots.cp.addPP(plot,pp)
  pp
}
