\name{iaxis}
\alias{iaxis}
\alias{iaxis.reset.all}
\title{
  Axis for a custom plot
}
\description{
  \code{iaxis} defines a new axis (aka scale) in a custom plot. The
  tasks of an axis is to map data coordinates to screen coordinates.

  \code{iaxis.reset.all} removes all axis definitions from a plot.
}
\usage{
  iaxis(type="cont", orientation="x", valuerange, pxc, variable=NULL,
        trl=0, drawAxis=FALSE)
  iaxis.reset.all(p=plot)
}
\arguments{
  \item{type}{type of the axis, can by any of \code{"continuous"} or
    \code{"discrete"} (or any non-ambiguous part hereof).}
  \item{orientation}{orientation, can be any of \code{"x"} or \code{"y"}}
  \item{valuerange}{range of the visible data values}
  \item{pxc}{corresponding range of the graphics coordinates}
  \item{variable}{variable associated with this axis (or \code{NULL} if
    this is an ad-hoc axis)}
  \item{trl}{translation orthogonal to the orientation if the axis is to
    be drawn (has no effect if \code{drawAxis} is \code{FALSE})}
  \item{drawAxis}{boolean value determining whether a line representing
    the axis should be drawn}
  \item{p}{plot}
}
\value{
  Resulting \code{iaxis} object which is a function that can be used
  to map data coordinates to graphical coordinates.
}
\details{
  This function is usually use in the \code{construct} part of a
  custom plot to setup the axes.

  Note that \code{iaxis.reset.all} is usually not needed, because all
  axes are removed automatically before a call to \code{construct}.
}
\seealso{
  \code{\link{icustom}}
}
\examples{
\dontrun{
  # called within construct():
  x <- iaxis("cont", "x", range(data[[1]]), c(20, width - 40))
  x.pos <- x(data[[1]])
}
}
\keyword{hplot}
